@@ -330,6 +330,12 @@ public:
 		 */
 		address();
 
+		/**
+		 * Copy constructor.
+		 * @param reference to object to copy from.
+		 */
+		address(const address& reference);
+
 		/**
 		 * Destroy address.  Deallocate addrinfo structure.
 		 */
@@ -438,7 +444,45 @@ public:
 		 * Add an individual socket address to our address list.
 		 * @param address to add.
 		 */
-		void add(struct sockaddr *address);
+		void add(sockaddr *address);
+
+		/**
+		 * Remove an individual socket address from our address list.
+		 * @param address to remove.
+		 * @return true if found and removed, false if not found.
+		 */
+		bool remove(struct sockaddr *address);
+
+		/**
+		 * Insert an individual socket address to our address list only if
+		 * unique.
+		 * @param address to insert into list.
+		 * @return true if inserted, false if duplicate.
+		 */
+		bool insert(struct sockaddr *address);
+
+		/**
+		 * Insert unique members from another socket address list to ours.
+		 * @param address list to insert into list.
+		 * @param address family to scan for or 0 for all.
+		 * @return count of addresses added.
+		 */
+		unsigned insert(struct addrinfo *address, int family = 0);
+
+		/**
+		 * Remove members from another socket address list from ours.
+		 * @param address list to remove from list.
+		 * @param address family to scan for or 0 for all.
+		 * @return count of addresses removed.
+		 */
+		unsigned remove(struct addrinfo *address, int family = 0);
+
+		/**
+		 * Copy an existing addrinfo into our object.  This is also used
+		 * to support the copy constructor.
+		 * @param address list to copy from.
+		 */
+		void copy(const struct addrinfo *address);
 
 		/**
 		 * Set an individual socket address for our address list.
@@ -453,6 +497,29 @@ public:
 		 * @param family of socket address.
 		 */
 		void set(const char *hostname, unsigned service = 0, int family = DEFAULT_FAMILY);
+
+		/**
+		 * Duplicate a socket address.
+		 * @param address to duplicate.
+		 * @return duplicate address object.
+		 */
+		static struct sockaddr *dup(struct sockaddr *address);
+
+		/**
+		 * Convert address object into ipv4 address.
+		 * @param address to convert.
+		 * @return new ipv4 address or NULL if not ipv4.
+		 */
+		static struct sockaddr_in *ipv4(struct sockaddr *address);
+
+#ifdef	AF_INET6
+		/**
+		 * Convert address object into ipv6 address.
+		 * @param address to convert.
+		 * @return new ipv6 address or NULL if not ipv6.
+		 */
+		static struct sockaddr_in6 *ipv6(struct sockaddr *address);
+#endif
 	};
 
 	friend class address;
@@ -1214,7 +1281,7 @@ public:
 	 * @return service port number.
 	 */
 	static short getservice(struct sockaddr *address);
-	
+
 	/**
 	 * Convert a socket address and service into a hash map index.
 	 * @param address to convert.
