@@ -51,6 +51,10 @@ shell::Option::Option(char shortopt, const char *longopt, bool value, const char
 	root = this;
 }
 
+shell::Option::~Option()
+{
+}
+
 #ifdef _MSWINDOWS_
 
 void shell::expand(void)
@@ -286,7 +290,6 @@ int shell::parse(int argc, char **argv)
 	char *arg;
 	char *argv0;
 	Option *node;
-	char optname[65];
 	unsigned len;
 	const char *value;
 	const char *opt;
