@@ -923,6 +923,12 @@ Socket::address::address()
 	list = NULL;
 }
 
+Socket::address::address(const address& from)
+{
+	list = NULL;
+	copy(from.list);
+}
+
 Socket::address::~address()
 {
 	clear();
@@ -1028,7 +1034,7 @@ struct sockaddr *Socket::address::getAddr(void)
 {
 	if(!list)
 		return NULL;
-	
+
 	return list->ai_addr;
 }
 
@@ -1067,6 +1073,133 @@ void Socket::address::set(struct sockaddr *addr)
 	add(addr);
 }
 
+bool Socket::address::remove(struct sockaddr *addr)
+{
+	assert(addr != NULL);
+	struct addrinfo *node = list, *prior = NULL;
+
+	while(node) {
+		if(node->ai_addr && equal(addr, node->ai_addr))
+			break;
+		prior = node;
+		node = node->ai_next;
+	}
+
+	if(!node)
+		return false;
+
+	if(!prior) 
+		list = node->ai_next;
+	else
+		prior->ai_next = node->ai_next;
+
+	node->ai_next = NULL;
+	freeaddrinfo(node);
+	return true;
+}
+
+unsigned Socket::address::insert(struct addrinfo *alist, int family)
+{
+	unsigned count = 0;
+	while(alist) {
+		if(!family || alist->ai_family == family) {
+			if(insert(alist->ai_addr))
+				++count;
+		}
+		alist = alist->ai_next;
+	}
+	return count;
+}
+
+unsigned Socket::address::remove(struct addrinfo *alist, int family)
+{
+	unsigned count = 0;
+	while(alist) {
+		if(!family || alist->ai_family == family) {
+			if(remove(alist->ai_addr))
+				++count;
+		}
+		alist = alist->ai_next;
+	}
+	return count;
+}
+
+bool Socket::address::insert(struct sockaddr *addr)
+{
+	assert(addr != NULL);
+
+	struct addrinfo *node = list;
+
+	while(node) {
+		if(node->ai_addr && equal(addr, node->ai_addr))
+			return false;
+		node = node->ai_next;
+	}
+
+	node = (struct addrinfo *)malloc(sizeof(struct addrinfo));
+	memset(node, 0, sizeof(node));
+	node->ai_family = addr->sa_family;
+	node->ai_addrlen = getlen(addr);
+	node->ai_next = list;
+	node->ai_addr = (struct sockaddr *)malloc(node->ai_addrlen);
+	memcpy(node->ai_addr, addr, node->ai_addrlen);
+	list = node;
+	return true;
+}
+
+void Socket::address::copy(const struct addrinfo *addr)
+{
+	struct addrinfo *last = NULL;
+	struct addrinfo *node;
+
+	clear();
+	while(addr) {
+		node = (struct addrinfo *)malloc(sizeof(struct addrinfo));
+		memcpy(node, addr, sizeof(struct addrinfo));
+		node->ai_next = NULL;
+		node->ai_addr = dup(addr->ai_addr);
+		if(last)
+			last->ai_next = node;
+		else
+			list = node;
+		last = node;
+	}
+}
+
+struct sockaddr_in *Socket::address::ipv4(struct sockaddr *addr)
+{
+	if(addr == NULL || addr->sa_family != AF_INET)
+		return NULL;
+
+	return (struct sockaddr_in*)addr;
+}
+
+#ifdef	AF_INET6
+struct sockaddr_in6 *Socket::address::ipv6(struct sockaddr *addr)
+{
+	if(addr == NULL || addr->sa_family != AF_INET6)
+		return NULL;
+
+	return (struct sockaddr_in6*)addr;
+}
+#endif			
+
+struct sockaddr *Socket::address::dup(struct sockaddr *addr)
+{
+	struct sockaddr *node;
+
+	if(!addr)
+		return NULL;
+
+	size_t len = getlen(addr);
+	if(!len)
+		return NULL;
+
+	node = (struct sockaddr *)malloc(len);
+	memcpy(node, addr, len);
+	return node;
+}	
+	
 void Socket::address::add(struct sockaddr *addr)
 {
 	assert(addr != NULL);
