@@ -24,6 +24,9 @@
 #include <stdarg.h>
 
 #ifndef	_MSWINDOWS_
+#include <netinet/tcp.h>
+#include <arpa/inet.h>
+#include <sys/socket.h>
 #include <sys/wait.h>
 #include <fcntl.h>
 #endif
@@ -290,7 +293,9 @@ void tcpstream::allocate(unsigned mss)
 {
 	unsigned size = mss;
 	unsigned max = 0;
+#ifdef	TCP_MAXSEG
 	socklen_t alen = sizeof(max);
+#endif
 	
 	if(mss == 1)
 		goto allocate;
@@ -548,14 +553,21 @@ void pipestream::open(const char *cmd, access_t mode, const char **envp, size_t
 	close();
 	
 	if(mode == RDONLY || mode == RDWR) {
-		pipe(input);
+		if(pipe(input))
+			return;
 		fsys::assign(rd, input[0]);
 	}
 	else
 		input[1] = ::open("/dev/null", O_RDWR);
 
 	if(mode == WRONLY || mode == RDWR) {
-		pipe(output);
+		if(pipe(output)) {
+			if(mode == RDWR) {
+				::close(input[0]);
+				::close(input[1]);
+			}
+			return;
+		}
 		fsys::assign(wr, output[1]);
 	}
 	else
