@@ -28,6 +28,10 @@ using namespace UCOMMON_NAMESPACE;
 const strsize_t string::npos = (strsize_t)(-1);
 const size_t memstring::header = sizeof(cstring);
 
+StringFormat::~StringFormat()
+{
+}
+
 string::cstring::cstring(strsize_t size)
 {
 	max = size;
@@ -825,7 +829,7 @@ void string::cut(strsize_t offset, strsize_t size)
 
 bool string::resize(strsize_t size)
 {
-	char fill;
+	char fill = 0;
 
 	if(!size) {
 		release();
